from .base import Tool


class DocIndexTool(Tool):
    """
    Document Index Tool, used to index document content into vector database
    """

    def __init__(self, **kwargs):
        pass

    def run(self, **kwargs):
        pass
