from .base import Tool


class CodeReplTool(Tool):
    """
    Code Repl Tool, used to execute code and return result
    """

    def __init__(self, **kwargs):
        pass

    def run(self, **kwargs):
        pass
