from .base import Tool


class RAGTool(Tool):
    """
    RAG retrieval tool, used to retrieve relevant documents from vector database
    """

    def __init__(self, **kwargs):
        pass

    def run(self, **kwargs):
        pass
